var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Creating Python UIs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Creating Python UIs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"creating-python-uis\"></a></div>\n\
<p>Autodesk 3ds Max ships with PySide 2.0, which is supported for both Python 2 and 3, and includes all standard PySide modules. PySide2 is the preferred framework for building UIs with Python in 3ds Max.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> Because 3ds Max is single-threaded, the main UI is not updated while a Python script is running, even if it changes the scene, unless a UI update is requested. For example, a new scene object will not appear in the viewport until the script exits or <span class=\'code\'>pymxs.runtime.redrawViews()</span> is called.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"getting-the-max-main-window-widget\"></a><h2 id=\"getting-the-max-main-window-widget\">Getting the Max Main Window Widget</h2></div>\n\
<p>In most cases you will need a parent widget for your PySide UI, which is the 3ds Max main window. The way to get the QWidget version of the 3ds Max main window is this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime\n\
from PySide2.QtWidgets import QWidget\n\
main_window_qwdgt = QWidget.find(runtime.windows.getMAXHWND())\n\
</pre></div><p>Widgets parented to the main widget will behave predictably, minimizing when the main window is minimized, and deleted properly when 3ds Max exits. To display the widget as modeless, call <span class=\'code\'>.show()</span>, and to open it as modal, call <span class=\'code\'>.exec_()</span>. Widgets parented to the QWidget version of the 3ds Max main window are not dockable.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> Because 3ds Max is single-threaded, the main UI is not updated while a Python script is running, even if it changes the scene, unless a UI update is requested. For example, a new scene object will not appear in the viewport until the script exits or <span class=\'code\'>pymxs.runtime.redrawViews()</span> is called.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"creating-a-dockable-widget\"></a><h2 id=\"creating-a-dockable-widget\">Creating a Dockable Widget</h2></div>\n\
<p>To enable docking (such as in the example below), you can use the <span class=\'code\'>QMainWindow</span> returned by the <span class=\'code\'>qtmax.GetMaxMainWindow()</span> convenience function.</p>\n\
<p>The following example shows how to create a dockable widget that is parented to and docks with the 3ds Max main window:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&#39;&#39;&#39;\n\
Demonstrates how to create a QDockWidget with PySide2 for use in 3ds Max\n\
&#39;&#39;&#39;\n\
\n\
from PySide2 import QtCore\n\
from PySide2 import QtGui\n\
from PySide2 import QtWidgets\n\
import qtmax\n\
\n\
from pymxs import runtime as rt\n\
\n\
def make_cylinder():\n\
    cyl = rt.Cylinder(radius=10, height=30)\n\
    rt.redrawViews()\n\
\n\
    return    \n\
    \n\
class PyMaxDockWidget(QtWidgets.QDockWidget):\n\
    def __init__(self, parent=None):\n\
        super(PyMaxDockWidget, self).__init__(parent)\n\
        self.setWindowFlags(QtCore.Qt.Tool)\n\
        self.setWindowTitle(&#39;Pyside Qt  Dock Window&#39;)\n\
        self.initUI()\n\
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)\n\
        \n\
    def initUI(self):\n\
        main_layout = QtWidgets.QVBoxLayout()\n\
        label = QtWidgets.QLabel(&quot;Click button to create a cylinder in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        cylinder_btn = QtWidgets.QPushButton(&quot;Cylinder&quot;)\n\
        cylinder_btn.clicked.connect(make_cylinder)\n\
        main_layout.addWidget(cylinder_btn)\n\
        widget = QtWidgets.QWidget()\n\
        widget.setLayout(main_layout)\n\
        self.setWidget(widget)\n\
        self.resize(250, 100)\n\
\n\
def main():\n\
    rt.resetMaxFile(rt.name(&#39;noPrompt&#39;))\n\
    main_window = qtmax.GetQMaxMainWindow()\n\
    w = PyMaxDockWidget(parent=main_window)\n\
    w.setFloating(True)\n\
    w.show()\n\
\n\
if __name__ == &#39;__main__&#39;:\n\
    main()\n\
</pre></div><p>Some notes on this example:</p>\n\
<ul>\n\
<li><p>Though not strictly necessary, this example sets the window flags to <span class=\'code\'>QtCore.Qt.Tool</span> to improve the appearance of the dialog, and sets the attribute <span class=\'code\'>QtCore.Qt.QA_DeleteOnClose</span> to delete the dialog immediately on close. Without this flag, the dialog is hidden, but not deleted when the close button is clicked.</p></li>\n\
<li><p>Objects created in your script need to be protected from being garbage collected. There are two approaches you can take. One is to keep references to them in a static class variable. The other is to inherit from a <span class=\'code\'>QWidget</span> class (in this example, <span class=\'code\'>QDockWidget</span>), which will hold the static reference, and since Qt now manages the lifecycle of the widget, it is protected from being garbage collected.</p></li>\n\
</ul>\n\
<div class=\'section\'><a id=\"loading-qt-ui-files\"></a><h2 id=\"loading-qt-ui-files\">Loading Qt UI Files</h2></div>\n\
<p>PySide2 provides a facility for loading UI files created by Qt Designer: <span class=\'code\'>QUiLoader</span>. This example looks for a ui file named <span class=\'code\'>test_ui.ui</span> located in the same directory as the running script and loads it. For more information, see <a href=\'https://doc.qt.io/qtforpython/PySide2/QtUiTools/QUiLoader.html#PySide2.QtUiTools.QUiLoader\' title=\'\' target=\'_blank\'>QUiLoader</a>.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import os\n\
from PySide2.QtWidgets import QVBoxLayout\n\
from PySide2.QtWidgets import QWidget\n\
from PySide2.QtWidgets import QDialog\n\
from PySide2.QtWidgets import QPushButton\n\
from PySide2.QtCore import QFile\n\
from PySide2.QtUiTools import QUiLoader\n\
from pymxs import runtime as rt\n\
\n\
\n\
\n\
class TestDialog(QDialog):\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        QDialog.__init__(self, parent)\n\
        loader = QUiLoader()\n\
        ui_file_path = os.path.join(  os.path.dirname(os.path.realpath(__file__)), &#39;test_ui.ui&#39;)\n\
        ui_file = QFile(ui_file_path)\n\
        ui_file.open(QFile.ReadOnly)\n\
        self.ui = loader.load(ui_file, self)\n\
        ui_file.close()\n\
        \n\
        layout = QVBoxLayout()\n\
        layout.addWidget(self.ui)\n\
        self.setLayout(layout)\n\
        btn = self.ui.findChild(QPushButton, &#39;pushButton&#39;)\n\
        btn.clicked.connect(self.makeTeapot)\n\
        \n\
    def makeTeapot(self):\n\
        rt.teapot()\n\
        rt.redrawViews()\n\
\n\
def main():\n\
    dlg = TestDialog()\n\
    dlg.show()\n\
\n\
\n\
if __name__ == &#39;__main__&#39;:\n\
    main()\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";