var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Threading in Python with 3ds Max</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Threading in Python with 3ds Max</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"threading-in-python-with-3ds-max\"></a></div>\n\
<p>3ds Max is a single-threaded application, which means that Python threads cannot interact with 3ds Max, either the UI or the MAXScript environment. 3ds Max is not &quot;thread safe&quot;. This means you cannot, in a thread use any pymxs calls (MAXScript runs on the main thread).</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> The <span class=\'code\'>pymxs.mxstoken()</span> function intended to support multi-threading is deprecated, and does not actually work.</p>\n\
</blockquote>\n\
<p>However, threading can still be used in your scripts for accomplishing tasks that would benefit from concurrency and/or parallelism, such as file IO. Both the standard <span class=\'code\'>threading</span> module and <span class=\'code\'>async</span> (in Python 3) can be used.</p>\n\
<div class=\'section\'><a id=\"usefulness-of-multi-threading\"></a><h2 id=\"usefulness-of-multi-threading\">Usefulness of multi threading</h2></div>\n\
<p>There are two main reasons to use multi threading in a program:</p>\n\
<ul>\n\
<li>to make good use of multiple CPUs (cores),</li>\n\
<li>and to do concurrent IO: multiple independent IO operations at the same time instead of sequentially.</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"making-use-of-multiple-cores\"></a><h3 id=\"making-use-of-multiple-cores\">Making Use of Multiple Cores</h3></div>\n\
<p>The way the Python Global Interpreter Lock (GIL) works makes it impossible for two python instructions to run at the same time on the same interpreter, no matter the number of cores available. The consequence of this is that python code executing tight computation loops, no matter the number of threads at work, will only keep the equivalent of one core busy.</p>\n\
<p>Heavy lifting code written in python does not benefit from multi threading, so what does? Sometimes, python programs call functions that were not implemented in python but in a different language (ex: C). The non-Python implementation of these library functions will typically release the GIL during their execution allowing threads to run normally, so that these native parts can be executed in parallel on multiple cores. So in essence, if the heavy lifting is initiated from Python but performed in another language (after the GIL has been released) the code may benefit from multiple cores. A good example of this scenario is <span class=\'code\'>SciPy</span> (<a href=\'https://scipy-cookbook.readthedocs.io/items/Multithreading.html\' title=\'\' target=\'_blank\'>see details here</a>).</p>\n\
<p>It is possible for native library functions to make good use of multiple cores (and GPUS, multiple machines, or any hardware available for that matter) even if the python code that makes the calls is single threaded. If for example a python program uses TensorFlow, this library will make good use of all the available hardware to operate in an efficient way ((<a href=\'https://www.tensorflow.org/guide/distributed_training\' title=\'\' target=\'_blank\'>explanations here</a>). It&#39;s the implementation of a given lengthy function that will distribute the work to the available hardware so that maximum parallelism can be achieved during the execution of this call.</p>\n\
<p>In short, what&#39;s important to understand is that the multi-threading in python may help to make better use of cores but only under very particular circumstances, and that some library functions are able to do that anyway without any help from python threads. In all situations, it depends on the internal implementation of external libraries.</p>\n\
<div class=\'section\'><a id=\"using-threads-for-concurrent-io\"></a><h2 id=\"using-threads-for-concurrent-io\">Using Threads for Concurrent IO</h2></div>\n\
<p>In most situations, performing independent IO operations concurrently takes less time than performing them sequentially, and this is the rationale for concurrent IO operations.</p>\n\
<p>Reading data from devices (such as a network or hard drive) or writing data to external devices takes time but does not require a lot of work from the CPU. When possible, it is beneficial to initiate multiple IO operations at the same time and process the data as it arrives. Normal (synchronous) IO functions block the calling thread until the IO operation completes. For that reason, initiating concurrent IO operations with blocking functions requires the use of multiple threads (the main thread launches multiple threads that each perform one blocking io operation).</p>\n\
<p>In 3ds Max, Python threads are useful in this scenario. As long as the worker threads don&#39;t use <span class=\'code\'>pymxs</span> (or objects created with <span class=\'code\'>pymxs</span>) this method can be used in 3ds Max to improve file IO performance.</p>\n\
<div class=\'section\'><a id=\"using-async-for-asynchronous-io\"></a><h2 id=\"using-async-for-asynchronous-io\">Using <span class=\'code\'>async</span> for Asynchronous IO</h2></div>\n\
<p>Threads can be used for achieving concurrent IO with blocking functions. Python 3 also includes the <span class=\'code\'>asyncio</span> library, which allows you to do the same thing without having to pay for the memory overhead and context switching of threads. Everything is executed on the same thread through the use of an event loop (using cooperative scheduling).</p>\n\
<p>The <span class=\'code\'>asyncio</span> library does not require threads, but it is possible to use threads with <span class=\'code\'>asyncio</span>. For example, it is possible to expose an operation running on a thread or a process as a future (see details in <span class=\'code\'>concurrent.futures</span>).</p>\n\
<p>In 3ds Max, it is possible to use <span class=\'code\'>asyncio</span> to perform concurrent IO without creating threads or processes. In this scenario only one thread is used so the Python code <em>can</em> (but shouldn&#39;t) call into <span class=\'code\'>pymxs</span>, because pymxs operations can themselves use blocking IO, and since these functions are not clearly identified and not awaitable, they will introduce synchronous IO to the mix.<br>This reduces the usefulness of asyncio, where every lengthy operation should be non-blocking.</p>\n\
<p>To play well with asyncio, pymxs would need to return its blocking functions as futures, but this is not currently the case.</p>\n\
<p>In python there are mechanisms that allow you to use thread-based operations or process-based operations as async operations. Calling from python threads into pymxs functions is not supported, no matter through which path this happens.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";