var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Accessing Object Properties and Controllers</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Accessing Object Properties and Controllers</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"accessing-object-properties-and-controllers\"></a></div>\n\
<p>Working with nested properties, such as controllers, in pymxs requires a different approach than in MAXScript. MAXScript &quot;knows&quot; about nested object properties, and does some extra parsing when it encounters them. Python and MAXScript treat properties indicated with dot notation differently: Python always returns a reference to the indicated property, while MAXScript returns a copy of the value. Attempting to assign a value may not work as intended in pymxs.</p>\n\
<blockquote>\n\
<p><strong>Note</strong>: There are some exceptions to this situation, as when the underlying property is a reference. So something like <span class=\'code\'>box.controller.position</span> will work, because controller is a reference.</p>\n\
</blockquote>\n\
<p>For example, in MAXScript you can do this:</p>\n\
<div class=\'codeBlock\'><pre>b = box()\n\
b.pos.x = 20\n\
</pre></div><p>The equivalent in pymxs does not work as intended, because <span class=\'code\'>b.pos.x</span> is returning a copy of the pos object, which is no longer connected to the box&#39;s position.</p>\n\
<p>There are three solutions to this problem:</p>\n\
<ol>\n\
<li>Use the MAXScript <span class=\'code\'>getProperty()</span> and <span class=\'code\'>setProperty()</span> functions</li>\n\
<li>Use the pymxs MXSWrapperBase <span class=\'code\'>getmxsprop()</span> and <span class=\'code\'>setmxsprop()</span> functions</li>\n\
<li>Work on a copy of the target property, and assign the object back</li>\n\
</ol>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
\n\
b.pos.x = 9  # trying to set the x value of the box position, but we got a copy of the position\n\
print(b.pos.x)  # see, we are still at pos.x == 0\n\
\n\
# solution: use MXSWrapper.setmxsprop()\n\
b.setmxsprop(&quot;pos.x&quot;, 9.0)  # properly set the nested property value\n\
print(b.pos.x)  # the x value of the box position is now at 9\n\
\n\
\n\
# OR use MAXScript setProperty()\n\
mxs.setProperty(b, &quot;pos.x&quot;, 10.0)\n\
print(mxs.getProperty(b, &quot;pos.x&quot;))\n\
\n\
# you can also do this:\n\
b_pos = b.pos\n\
b_pos.x = 10  # set the copied value from the box property\n\
b.pos = b_pos  # properly apply back the changed pos value to the box\n\
print(b.pos.x)  # the x value of the box position is now at 10\n\
\n\
# getting the position:\n\
print(b.pos.x)  # 10.0 - fetches the proper value, but it is a copy of the position\n\
print(b.getmxsprop(&quot;pos.x&quot;)) # 10.0 - fetches the proper value\n\
print(mxs.getProperty(b, &quot;pos.x&quot;)) # 10.0 - also works\n\
</pre></div><div class=\'section\'><a id=\"working-with-controllers\"></a><h2 id=\"working-with-controllers\">Working with Controllers</h2></div>\n\
<p>Here is an example of some MAXScript code:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-mxs\">myTeapot = teapot()\n\
myTeapot.pos.controller = noise_position()\n\
</pre></div><p>In MAXScript, as in Python, <span class=\'code\'>myTeapot.pos</span> is a Point3, which doesn&#39;t have an associated controller property. Attempting to get <span class=\'code\'>myTeapot.pos.controller</span> in Python results in an error such as: <span class=\'code\'>AttributeError: &#39;pymxs.MXSWrapperBase&#39; object has no attribute &#39;controller&#39;</span>.</p>\n\
<p>But in MAXScript <span class=\'code\'>myTeapot.pos.controller</span> is perfectly valid, as it evaluates to an object structure hierarchy. Therefore, to achieve the same thing using pymxs, we must use a different approach and use <span class=\'code\'>getPropertyController()</span> / <span class=\'code\'>setPropertyController()</span>.</p>\n\
<p>Here&#39;s a complete example where we set the Position XYZ controller of a box, and the X component of the position for a sphere:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
# set the transform -&gt; position controller\n\
t = rt.teapot()\n\
rt.setPropertyController( t.controller, &#39;Position&#39;, rt.noise_position())\n\
print(rt.getPropertyController(t.controller, &#39;Position&#39;))\n\
\n\
# set the transform -&gt; position -&gt; x_position controller\n\
s = rt.sphere()\n\
sphr_pos_ctrl = rt.getPropertyController(s.controller, &#39;Position&#39;)\n\
rt.setPropertyController(sphr_pos_ctrl, &#39;X Position&#39;, rt.noise_float())\n\
print(rt.getPropertyController(sphr_pos_ctrl, &#39;X Position&#39;))\n\
</pre></div><p>Output:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">Controller:Noise_Position\n\
Controller:Noise_Float\n\
</pre></div><div class=\'section\'><a id=\"expression-controllers\"></a><h2 id=\"expression-controllers\">Expression Controllers</h2></div>\n\
<p>Expression controllers are created and set in the same way, though they require the extra steps of adding an expression and (if required) scalar and vector targets. In this example, we add an expression controller to the X Position sub-controller of an object&#39;s position controller, so that its X position always matches a target object.</p>\n\
<p>Note: The name of the controller you pass to getPropertyController() is the same string that appears in the <strong>Track View - Curve Editor</strong> in the 3ds Max UI.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
# target:\n\
s = rt.sphere()\n\
\n\
t = rt.teapot(pos=rt.point3(20,20,0))\n\
s_pos_ctl = rt.getPropertyController(s.controller, &#39;Position&#39;)\n\
expr_pos_ctl = rt.position_expression()\n\
# first add the scalar variable targeting the sphere&#39;s X position\n\
expr_pos_ctl.addScalarTarget(&#39;sphere_x&#39;, rt.getPropertyController(s_pos_ctl, &#39;X Position&#39;))\n\
# then add the expression that uses that variable\n\
expr_pos_ctl.setExpression(&#39;[sphere_x, 20,0]&#39;)\n\
# finally, set the controller\n\
rt.setPropertyController(t.controller, &quot;Position&quot;, expr_pos_ctl)\n\
\n\
</pre></div><div class=\'section\'><a id=\"assigning-controllers-where-there-is-not-an-existing-controller\"></a><h2 id=\"assigning-controllers-where-there-is-not-an-existing-controller\">Assigning Controllers Where There is not an Existing Controller</h2></div>\n\
<p>The transform properties of an object has default controllers, and can be accessed and modified as explained above. Adding a controller to a property that does not currently have one with pymxs is not as straightforward, because there is no name associated with the property&#39;s controller.</p>\n\
<p>3ds Max stores all the controllers for properties for an object in a structure called a <em>subanim</em> (sub-animatable). A subanim can contain nested subanims. To assign a controller to a property that does not have a controller, we need to find the property&#39;s location in the subanim tree structure by index, and assign the controller to the result. We can do this using the MAXScript function <span class=\'code\'>getSubanim()</span>.</p>\n\
<p>For example, to assign a float controller to an object&#39;s <strong>width</strong> property:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
bw= mxs.getSubAnim(mxs.getSubAnim(b, 4), 2)\n\
mxs.setProperty(bw, &#39;controller&#39;, mxs.bezier_float())\n\
</pre></div><p>This gets the box node&#39;s fourth subAnim (the box object), and then gets the second subAnim (the width property).</p>\n\
<p>Note that the <span class=\'code\'>getSubAnim()</span> function gets the subAnim, while <span class=\'code\'>getPropertyController()</span> (illustrated above) gets the controller for the subAnim. So to expand the previous example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
# get the subAnim for X_Rotation:\n\
brx = mxs.getSubAnim(mxs.getSubAnim(mxs.getSubAnim(b, 3), 2), 1) \n\
# get the controller for X_Rotation:\n\
brxc = mxs.getPropertyController(mxs.getPropertyController(b.controller, &#39;Rotation&#39;), &#39;X Rotation&#39;)\n\
brx.controller == brxc # this is True\n\
</pre></div><p>This example illustrates visiting all the subAnims in an object&#39;s subAnim tree, and prints out the corresponding names and indices. It can be used to determine the correct indices to use to assign new controllers:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as mxs\n\
\n\
b = mxs.box()\n\
\n\
for sub_i in range(b.numsubs):\n\
    sub = mxs.getSubAnim(b, sub_i+1)\n\
    print(f&#39;{sub.name} [{sub_i+1}]&#39;)\n\
    \n\
    if sub.controller == None:\n\
        print(&#39; No controller assigned to it&#39;)\n\
    else:\n\
        print(f&#39; current controller: {(sub.controller)}&#39;)\n\
    if sub.keys == None:\n\
        print(&#39; No keys assigned to it&#39;)\n\
\n\
    for secsub_i in range(sub.numsubs):\n\
        secsub = mxs.getSubAnim(sub, secsub_i+1)\n\
        print(f&#39;\\t{secsub.name} [{secsub_i+1}]&#39;)\n\
\n\
        if secsub.controller == None:\n\
            print(&#39;\\t No controller assigned to it&#39;)\n\
        else:\n\
            print(f&#39;\\t current controller: {(secsub.controller)}&#39;)\n\
        if secsub.keys == None:\n\
            print(&#39;\\t No keys assigned to it&#39;)\n\
        for thirdsub_i in range(secsub.numsubs):\n\
            thirdsub = mxs.getSubAnim(secsub, thirdsub_i+1)\n\
            print(f&#39;\\t\\t{thirdsub.name} [{thirdsub_i+1}]&#39;)\n\
            if secsub.controller == None:\n\
                print(&#39;\\t No controller assigned to it&#39;)\n\
            else:\n\
                print(f&#39;\\t\\t current controller: {(thirdsub.controller)}&#39;)\n\
</pre></div><p>This displays the following output:</p>\n\
<p><span class=\'code\'>Visibility [1]  No controller assigned to it  No keys assigned to it Space Warps [2]  No controller assigned to it  No keys assigned to it Transform [3]  current controller: Controller:Position_Rotation_Scale     Position [1]      current controller: Controller:Position_XYZ         X Position [1]          current controller: Controller:Bezier_Float         Y Position [2]          current controller: Controller:Bezier_Float         Z Position [3]          current controller: Controller:Bezier_Float     Rotation [2]      current controller: Controller:Euler_XYZ         X Rotation [1]          current controller: Controller:Bezier_Float         Y Rotation [2]          current controller: Controller:Bezier_Float         Z Rotation [3]          current controller: Controller:Bezier_Float     Scale [3]      current controller: Controller:Bezier_Scale Object (Box) [4]  No controller assigned to it  No keys assigned to it     Length [1]      No controller assigned to it      No keys assigned to it     Width [2]      No controller assigned to it      No keys assigned to it     Height [3]      No controller assigned to it      No keys assigned to it     Width Segments [4]      No controller assigned to it      No keys assigned to it     Length Segments [5]      No controller assigned to it      No keys assigned to it     Height Segments [6]      No controller assigned to it      No keys assigned to it Material [5]  No controller assigned to it  No keys assigned to it Image Motion Blur Multiplier [6]  No controller assigned to it  No keys assigned to it Object Motion Blur On Off [7]  No controller assigned to it  No keys assigned to it</span></p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";